from awsume import __data__

__VERSION__ = __data__.version
__NAME__ = __data__.name
__AUTHOR__ = __data__.author
__AUTHOR_EMAIL__ = __data__.author_email
__DESCRIPTION__ = __data__.description
__LICENSE__ = __data__.license
__HOMEPAGE__ = __data__.homepage
__MESSAGE__ = __data__.message
